@@ -34,9 +34,9 @@
 #include <klocale.h>
 #include <kpopupmenu.h>
 #include <kstdaction.h>
+#include <kstdaccel.h>
 #include <ktempfile.h>
 #include <kio/scheduler.h>
-#include <kkeydialog.h>
 #include <kaboutdata.h>
 
 
@@ -77,12 +77,15 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
                   QObject* parent, const char* name,
                   const QStringList &args ) :
     KParts::ReadOnlyPart( parent, name ),
+    _fitTimer( new QTimer( this ) ),
     _job( 0 ),
     _mimetypeScanner( 0 ),
     _dirtyHandler( new QTimer( this ) ),
     _isGuiInitialized( false ),
     _isFileDirty( false ),
-    _stickyOptions( false )
+    _stickyOptions( false ),
+    _embeddedInKGhostView( !args.contains( "KParts::ReadOnlyPart" ) ),
+    _customZoomIndex( -1 )
 {
     setInstance( KGVFactory::instance() );
 
@@ -253,25 +256,25 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
     for ( QValueList<double>::iterator first = mags.begin(), last = mags.end();
 	    first != last;
 	    ++first ) {
-	translated << QString( i18n( "zoom-factor (percentage)", "%1%" ) ).arg( *first * 100.0 );
+	translated << i18n( "zoom-factor (percentage)", "%1%" ).arg( *first * 100.0 );
 	if ( *first == 1.0 ) idx = cur;
 	++cur;
     }
     _zoomTo->setItems( translated );
     _zoomTo->setCurrentItem( idx );
 
+    _fitWidth = new KAction( i18n( "&Fit to Page Width" ), 0, this,
+	    SLOT( slotFitToPage() ), actionCollection(),
+	    "fit_to_page");
+    _fitScreen = new KAction( i18n( "&Fit to Screen" ), Key_S, this,
+	    SLOT( slotFitToScreen() ), actionCollection(),
+	    "fit_to_screen");
 
-    /*
-    _fitWidth = new KAction( i18n( "Fit Width" ),
-                             0, this, SLOT( slotFitWidth() ),
-                             actionCollection(), "fitWidth" );
-    */
-
-    _prevPage  = KStdAction::prior( this, SLOT( slotPrevPage() ),
+    _prevPage  = new KAction( i18n( "Previous Page" ), CTRL+Key_PageUp, this, SLOT( slotPrevPage() ),
                                     actionCollection(), "prevPage" );
     _prevPage->setWhatsThis( i18n( "Moves to the previous page of the document" ) );
 
-    _nextPage  = KStdAction::next( this, SLOT( slotNextPage() ),
+    _nextPage  = new KAction( i18n( "Next Page" ), CTRL + Key_PageDown, this, SLOT( slotNextPage() ),
                                    actionCollection(), "nextPage" );
     _nextPage->setWhatsThis( i18n( "Moves to the next page of the document" ) );
 
@@ -283,12 +286,16 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
                                    actionCollection(), "goToEnd" );
     _lastPage->setWhatsThis( i18n( "Moves to the last page of the document" ) );
 
+    KShortcut readUpShort = KStdAccel::shortcut( KStdAccel::Prior );
+    readUpShort.append( KKey( SHIFT+Key_Space ) );
     _readUp    = new KAction( i18n( "Read Up Document" ), "previous",
-                              SHIFT+Key_Space, this, SLOT( slotReadUp() ),
+                              readUpShort, this, SLOT( slotReadUp() ),
                               actionCollection(), "readUp" );
-    // don't specify Key_Space as Accel, it breaks the lineedit in konq! (Simon)
+
+    KShortcut readDownShort = KStdAccel::shortcut( KStdAccel::Next );
+    readDownShort.append( KKey( Key_Space ) );
     _readDown  = new KAction( i18n( "Read Down Document" ), "next",
-                              /*Key_Space*/0, this, SLOT( slotReadDown() ),
+                              readDownShort, this, SLOT( slotReadDown() ),
                               actionCollection(), "readDown" );
 
     _gotoPage = KStdAction::gotoPage( _docManager, SLOT( goToPage() ),
@@ -297,17 +304,22 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
     //-- Settings Menu ------------------------------------------------------
     _showScrollBars = new KToggleAction( i18n( "Show &Scrollbars" ), 0,
                                 actionCollection(), "show_scrollbars" );
+    _showScrollBars->setCheckedState(i18n("Hide &Scrollbars"));
     _watchFile	    = new KToggleAction( i18n( "&Watch File" ), 0,
                                 this, SLOT( slotWatchFile() ),
                                 actionCollection(), "watch_file" );
     _showPageList   = new KToggleAction( i18n( "Show &Page List" ), 0,
                                 actionCollection(), "show_page_list" );
+    _showPageList->setCheckedState(i18n("Hide &Page List"));
     _showPageLabels = new KToggleAction( i18n("Show Page &Labels"), 0,
                                 actionCollection(), "show_page_labels" );
+    _showPageLabels->setCheckedState(i18n("Hide Page &Labels"));
+    KStdAction::preferences( this, SLOT( slotConfigure() ), actionCollection() );
+    /*
     new KAction( i18n( "&Configure KGhostView..." ), "configure", 0,
                  this, SLOT( slotConfigure() ),
-                 actionCollection(), "configure" );
-    KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
+		 actionCollection(), "configure" );
+      */
     connect( _showScrollBars, SIGNAL( toggled( bool ) ),
              SLOT( showScrollBars( bool ) ) );
     connect( _showPageList, SIGNAL( toggled( bool ) ),
@@ -331,11 +343,13 @@ KGVPart::KGVPart( QWidget* parentWidget, const char*,
     connect( _pageView, SIGNAL( ReadUp() ), SLOT( slotReadUp() ));
     connect( _pageView, SIGNAL( ReadDown() ), SLOT( slotReadDown() ));
 
-    readSettings();
-
     QStringList items = document()->mediaNames();
     items.prepend( i18n( "Auto ") );
     _selectMedia->setItems( items );
+
+    readSettings();
+
+    updatePageDepActions();
 }
 
 KGVPart::~KGVPart()
@@ -345,16 +359,11 @@ KGVPart::~KGVPart()
     writeSettings();
 }
 
-void KGVPart::slotConfigureKeys()
-{
-    KKeyDialog::configure( actionCollection(), this );
-}
-
 KAboutData* KGVPart::createAboutData()
 {
     KAboutData* about = new KAboutData( "kghostview", I18N_NOOP( "KGhostView"),
             KGHOSTVIEW_VERSION,
-            I18N_NOOP( "Viewer for PostScript (.ps, .eps) and Portable Document Format (.pdf) files."),
+            I18N_NOOP( "Viewer for PostScript (.ps, .eps) and Portable Document Format (.pdf) files"),
             KAboutData::License_GPL,
             "(C) 1998 Mark Donohoe, (C) 1999-2000 David Sweet, "
             "(C) 2000-2003 Wilco Greven",
@@ -413,6 +422,7 @@ bool KGVPart::closeURL()
     if( !m_file.isEmpty() )
         _fileWatcher->removeFile( m_file );
     _mimetype = QString::null;
+    updatePageDepActions();
     stateChanged( "initState" );
     return KParts::ReadOnlyPart::closeURL();
 }
@@ -424,6 +434,8 @@ void KGVPart::writeSettings()
     general.writeEntry( "WatchFile", _watchFile->isChecked() );
     general.writeEntry( "ShowPageList", _showPageList->isChecked() );
     general.writeEntry( "ShowPageNames", _showPageLabels->isChecked() );
+    if ( !_embeddedInKGhostView )
+        general.writeEntry( "Display Options", DisplayOptions::toString( miniWidget()->displayOptions() ) );
     general.sync();
 }
 
@@ -440,6 +452,11 @@ void KGVPart::readSettings()
     showPageLabels( _showPageLabels->isChecked() );
 
     _showLogWindow = _configDialog->showMessages();
+    if ( !_embeddedInKGhostView ) {
+        DisplayOptions options;
+        if ( DisplayOptions::fromString( options, general.readEntry( "Display Options" ) ) )
+            setDisplayOptions( options );
+    }
 }
 
 void KGVPart::slotScrollLeft()
@@ -486,32 +503,14 @@ void KGVPart::slotReadDown()
 
 void KGVPart::slotPrevPage()
 {
-    if( !( document() && document()->isOpen() ) )
-	return;
-
+    if( !document() || !document()->isOpen() ) return;
     _docManager->prevPage();
-    // I am commenting the line below in response to 58514
-    // I am afraid that someone will dislike the new behaviour, though :-)
-    //
-    // BTW, this seems to mirror kdvi, although I admit I don't use that app
-    // much, so I could be wrong.
-    //
-    // If no one does, then the comment should be reverted.
-    // LPC (27 Jun 2003)
-    //_pageView->scrollTop();
 }
 
 void KGVPart::slotNextPage()
 {
-
-    if( !( document() && document()->isOpen() ) )
-	return;
-    if( _docManager->nextPage() ) {
-	// See above
-	//_pageView->scrollTop();
-    }
-    else
-	_pageView->scrollBottom();
+    if( !document() || !document()->isOpen() ) return;
+    _docManager->nextPage();
 }
 
 void KGVPart::slotGotoStart()
@@ -559,7 +558,14 @@ void KGVPart::updateZoomActions()
     _zoomOut->setEnabled(!_docManager->atMinZoom());
     _zoomTo->setEnabled( true );
     QStringList items = _zoomTo->items();
-    double zoom = miniWidget()->displayOptions().magnification();
+    bool updateItems = false;
+    if (_customZoomIndex != -1)
+    {
+        items.remove(items.at(_customZoomIndex));
+        _customZoomIndex = -1;
+        updateItems = true;
+    }
+    double zoom = floor(miniWidget()->displayOptions().magnification()*1000.0) / 10.0;
     unsigned idx = 0;
     for ( QStringList::iterator first = items.begin(), last = items.end();
 	    first != last;
@@ -569,31 +575,39 @@ void KGVPart::updateZoomActions()
 	cur = cur.simplifyWhiteSpace();
 	bool ok = false;
 	double z = cur.toDouble(&ok);
-	if ( ok && std::abs( z - zoom * 100.0 ) < 0.1 ) {
-	    _zoomTo->setCurrentItem( idx );
-	    return;
+	if ( ok ) {
+            if (std::abs( z - zoom ) < 0.1 ) {
+                if (updateItems)
+                    _zoomTo->setItems( items );
+	        _zoomTo->setCurrentItem( idx );
+	        return;
+            }
+            if ( z > zoom )
+                break;
 	}
 	++idx;
     }
-    /* TODO:
-     *
-     * One should reorder the _zoomTo->items() list here.
-     * This is because any values the user types in himself will get
-     * put at the end of the list, which is not optimal.
-     *
-     * LPC 19 Jun 2003
-     */
+
+    // Show percentage that isn't predefined
+    items.insert( items.at(idx), 1, i18n( "zoom-factor (percentage)", "%1%" ).arg( zoom ) );
+    _zoomTo->setItems( items );
+    _zoomTo->setCurrentItem( idx );
+    _customZoomIndex = idx;
 }
 
 void KGVPart::updatePageDepActions()
 {
-    if( !( document() && document()->isOpen() ) )
-	return;
+    bool hasDoc = document() && document()->isOpen();
 
-    _prevPage->setEnabled( !_docManager->atFirstPage() );
-    _firstPage->setEnabled( !_docManager->atFirstPage() );
-    _nextPage->setEnabled( !_docManager->atLastPage() );
-    _lastPage->setEnabled( !_docManager->atLastPage() );
+    _fitWidth->setEnabled( hasDoc );
+    _fitScreen->setEnabled( hasDoc );
+
+    _prevPage->setEnabled( hasDoc && !_docManager->atFirstPage() );
+    _firstPage->setEnabled( hasDoc && !_docManager->atFirstPage() );
+    _nextPage->setEnabled( hasDoc && !_docManager->atLastPage() );
+    _lastPage->setEnabled( hasDoc && !_docManager->atLastPage() );
+    _gotoPage->setEnabled( hasDoc &&
+          !(_docManager->atFirstPage() && _docManager->atLastPage()) );
 
     updateReadUpDownActions();
 }
@@ -601,7 +615,11 @@ void KGVPart::updatePageDepActions()
 void KGVPart::updateReadUpDownActions()
 {
     if( !( document() && document()->isOpen() ) )
+    {
+	_readUp->setEnabled( false );
+	_readDown->setEnabled( false );
 	return;
+    }
 
     if( _docManager->atFirstPage() && _pageView->atTop() )
 	_readUp->setEnabled( false );
@@ -698,7 +716,6 @@ void KGVPart::slotOpenFileCompleted()
 	if ( !_fileWatcher->contains( m_file ) )
 	    _fileWatcher->addFile( m_file );
 	slotWatchFile();
-	slotZoom( i18n( "zoom-factor (percentage)", "%1%" ). arg( 100 ) );
 	updateZoomActions();
 	emit completed();
     }
@@ -714,12 +731,14 @@ void KGVPart::slotGhostscriptOutput( char* data, int len )
 
 void KGVPart::slotGhostscriptError( const QString& error )
 {
-    // FIXME: find a significant error message
-    _logWindow->setLabel( i18n( "An error occurred in rendering.\n%1\n%2\n" )
+    _logWindow->setLabel( i18n( "<qt>An error occurred in rendering.<br>"
+				"<strong>%1</strong><br>"
+				"The display may contain errors.<br>"
+				"Below are any error messages which were received from Ghostscript "
+				"(<nobr><strong>%2</strong></nobr>) "
+				"which may help you.</qt>" )
 		    .arg( error )
-		    .arg( i18n( "You might see some errors in the display or it may work.\n"
-			    "The error messages below are from Ghostscript (\"gs\") and may "
-			    "help identify or correct the problem.\n" ) ),
+		    .arg( configDialog()->interpreterPath() ) ,
 	   true );
     // The true above makes it show a "configure gs" option, but maybe we
     // should trigger an auto-redetection?
@@ -878,6 +897,35 @@ void KGVPart::slotZoom( const QString& nz )
     miniWidget()->setDisplayOptions( options );
     miniWidget()->redisplay();
     _mainWidget->setFocus();
+    updateZoomActions();
+}
+
+void KGVPart::slotFitToPage()
+{
+    kdDebug(4500) << "KGVPart::slotFitToPage()" << endl;
+    if( pageView()->page() )
+	miniWidget()->fitWidth( pageView()->viewport()->width() - 16 );
+    // We subtract 16 pixels because of the page decoration.
+    updateZoomActions();
+}
+
+void KGVPart::slotFitToScreen()
+{
+    kdDebug(4500) << "KGVPart::slotFitToScreen()" << endl;
+    if ( _fitTimer->isActive() ) {
+	disconnect( _fitTimer, SIGNAL( timeout() ), this, 0 );
+	connect( _fitTimer, SIGNAL( timeout() ), SLOT( slotDoFitToScreen() ) );
+    }
+    else slotDoFitToScreen();
+}
+
+void KGVPart::slotDoFitToScreen()
+{
+    kdDebug(4500) << "KGVPart::slotDoFitToScreen()" << endl;
+    if( pageView()->page() )
+	miniWidget()->fitWidthHeight( pageView()->viewport()->width() - 16,
+					pageView()->viewport()->height() - 16 );
+    updateZoomActions();
 }
 
 void KGVPart::reloadFile()
@@ -945,13 +993,8 @@ void KGVRun::foundMimeType( const QString& mimetype )
 
     if( m_job && m_job->inherits( "KIO::TransferJob" ) )
     {
-	/* The code below should be correct except for what seems to be a bug in kio.
-	 * See bug:69456 for details
 	KIO::TransferJob *job = static_cast< KIO::TransferJob* >( m_job );
 	job->putOnHold();
-	KIO::Scheduler::publishSlaveOnHold();
-	*/
-	m_job->kill(true);
 	m_job = 0;
     }
 
@@ -964,13 +1007,8 @@ void KGVRun::foundMimeType( const QString& mimetype )
 
 void KGVPart::updateFullScreen( bool fs )
 {
-    if ( fs ) {
-	_markList->hide();
-	_scrollBox->hide();
-    } else {
-	_markList->show();
-	_scrollBox->show();
-    }
+    if ( fs ) showMarkList( false );
+    else showMarkList( _showPageList->isChecked() );
 }
 
 void KGVPart::showPopup( int, int, const QPoint& pos )
