@@ -16,6 +16,7 @@
 #include <kparts/genericfactory.h>
 #include <kurldrag.h>
 #include <kinputdialog.h>
+#include <ktempfile.h>
 
 #include "part.h"
 
@@ -24,8 +25,8 @@
 
 #include "GlobalParams.h"
 #include "PDFDoc.h"
-#include "XOutputDev.h"
-#include "QOutputDevKPrinter.h"
+#include "PSOutputDev.h"
+#include "QOutputDevPixmap.h"
 
 // #include "kpdf_canvas.h"
 #include "kpdf_pagewidget.h"
@@ -35,6 +36,8 @@ K_EXPORT_COMPONENT_FACTORY(libkpdfpart, KPDFPartFactory)
 
 using namespace KPDF;
 
+unsigned int Part::m_count = 0;
+
 Part::Part(QWidget *parentWidget, const char *widgetName,
            QObject *parent, const char *name,
            const QStringList & /*args*/ )
@@ -47,14 +50,14 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
   new BrowserExtension(this);
 
   globalParams = new GlobalParams("");
+  globalParams->setupBaseFonts(NULL);
 
   // we need an instance
   setInstance(KPDFPartFactory::instance());
 
-  pdfpartview = new PDFPartView(parentWidget, widgetName);
+  pdfpartview = new PDFPartView(parentWidget, widgetName, &m_docMutex);
 
-  connect(pdfpartview->pagesListBox, SIGNAL( clicked ( QListBoxItem * ) ),
-          this, SLOT( pageClicked ( QListBoxItem * ) ));
+  connect(pdfpartview, SIGNAL( clicked ( int ) ), this, SLOT( pageClicked ( int ) ));
 
   m_outputDev = pdfpartview->outputdev;
   m_outputDev->setAcceptDrops( true );
@@ -64,20 +67,20 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
 
   m_showScrollBars = new KToggleAction( i18n( "Show &Scrollbars" ), 0,
                                        actionCollection(), "show_scrollbars" );
+  m_showScrollBars->setCheckedState(i18n("Hide &Scrollbars"));
   m_showPageList   = new KToggleAction( i18n( "Show &Page List" ), 0,
                                        actionCollection(), "show_page_list" );
+  m_showPageList->setCheckedState(i18n("Hide &Page List"));
   connect( m_showScrollBars, SIGNAL( toggled( bool ) ),
              SLOT( showScrollBars( bool ) ) );
   connect( m_showPageList, SIGNAL( toggled( bool ) ),
              SLOT( showMarkList( bool ) ) );
 
   // create our actions
-#if 0
   KStdAction::find    (this, SLOT(find()),
                        actionCollection(), "find");
   KStdAction::findNext(this, SLOT(findNext()),
                        actionCollection(), "find_next");
-#endif
   m_fitToWidth = new KToggleAction(i18n("Fit to Page &Width"), 0,
                        this, SLOT(slotFitToWidthToggled()),
                        actionCollection(), "fit_to_width");
@@ -86,14 +89,11 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
   KStdAction::zoomOut (m_outputDev, SLOT(zoomOut()),
                        actionCollection(), "zoom_out");
 
-#if 0
   KStdAction::back    (this, SLOT(back()),
                        actionCollection(), "back");
   KStdAction::forward (this, SLOT(forward()),
                        actionCollection(), "forward");
-#endif
 
-  KStdAction::print( this, SLOT( print() ), actionCollection() );
   KStdAction::printPreview( this, SLOT( printPreview() ), actionCollection() );
 
   m_prevPage = KStdAction::prior(this, SLOT(slotPreviousPage()),
@@ -123,11 +123,16 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
   m_zoomTo->clear();
 
   QStringList translated;
+  QString localValue;
+  QString double_oh("00");
   int idx = 0;
   int cur = 0;
   for ( int i = 0; i < 10;i++)
   {
-      translated << QString( "%1%" ).arg( zoomValue[i] * 100.0 );
+      localValue = KGlobal::locale()->formatNumber( zoomValue[i] * 100.0, 2 );
+      localValue.remove( KGlobal::locale()->decimalSymbol()+double_oh );
+
+      translated << QString( "%1%" ).arg( localValue );
       if ( zoomValue[i] == 1.0 )
           idx = cur;
       ++cur;
@@ -147,11 +152,14 @@ Part::Part(QWidget *parentWidget, const char *widgetName,
   connect( m_outputDev, SIGNAL( spacePressed() ), this, SLOT( slotReadDown() ) );
   readSettings();
   updateActionPage();
+  m_count++;
 }
 
 Part::~Part()
 {
-    delete globalParams;
+    m_count--; 
+    pdfpartview->stopThumbnailGeneration();
+    if (m_count == 0) delete globalParams; 
     writeSettings();
 }
 
@@ -160,9 +168,14 @@ void Part::slotZoom( const QString&nz )
     QString z = nz;
     double zoom;
     z.remove(  z.find(  '%' ), 1 );
-    zoom = KGlobal::locale()->readNumber(  z ) / 100;
-    kdDebug() << "ZOOM = "  << nz << ", setting zoom = " << zoom << endl;
-    m_outputDev->zoomTo( zoom );
+    bool isNumber = true;
+    zoom = KGlobal::locale()->readNumber(  z, &isNumber ) / 100;
+ 
+    if ( isNumber )
+    {
+    	kdDebug() << "ZOOM = "  << nz << ", setting zoom = " << zoom << endl;
+    	m_outputDev->zoomTo( zoom );
+    }
 }
 
 void Part::slotGoToPage()
@@ -170,7 +183,7 @@ void Part::slotGoToPage()
     if ( m_doc )
     {
         bool ok = false;
-        int num = KInputDialog::getInteger(i18n("Go to Page"), i18n("Page:"), m_currentPage,
+        int num = KInputDialog::getInteger(i18n("Go to Page"), i18n("Page:"), m_currentPage+1,
                                            1, m_doc->getNumPages(), 1, 10, &ok/*, _part->widget()*/);
         if (ok)
             goToPage( num );
@@ -179,9 +192,9 @@ void Part::slotGoToPage()
 
 void Part::goToPage( int page )
 {
-    m_currentPage = page;
-    pdfpartview->pagesListBox->setCurrentItem(m_currentPage-1);
-    m_outputDev->setPage(m_currentPage);
+    m_currentPage = page-1;
+    pdfpartview->setCurrentItem(m_currentPage);
+    m_outputDev->setPage(m_currentPage+1);
     updateActionPage();
 }
 
@@ -192,10 +205,7 @@ void Part::slotOpenUrlDropped( const KURL &url )
 
 void Part::setFullScreen( bool fs )
 {
-    if ( !fs )
-        pdfpartview->pagesListBox->show();
-    else
-        pdfpartview->pagesListBox->hide();
+    pdfpartview->showPageList(!fs);
 }
 
 
@@ -204,9 +214,9 @@ void Part::updateActionPage()
     if ( m_doc )
     {
         m_firstPage->setEnabled(m_currentPage!=0);
-        m_lastPage->setEnabled(m_currentPage<m_doc->getNumPages());
+        m_lastPage->setEnabled(m_currentPage<m_doc->getNumPages()-1);
         m_prevPage->setEnabled(m_currentPage!=0);
-        m_nextPage->setEnabled(m_currentPage<m_doc->getNumPages());
+        m_nextPage->setEnabled(m_currentPage<m_doc->getNumPages()-1);
     }
     else
     {
@@ -263,19 +273,16 @@ void Part::showScrollBars( bool show )
 
 void Part::showMarkList( bool show )
 {
-    if ( show )
-        pdfpartview->pagesListBox->show();
-    else
-        pdfpartview->pagesListBox->hide();
+    pdfpartview->showPageList(show);
 }
 
 void Part::slotGotoEnd()
 {
     if ( m_doc && m_doc->getNumPages() > 0 );
     {
-        m_currentPage = m_doc->getNumPages();
-        m_outputDev->setPage(m_currentPage);
-        pdfpartview->pagesListBox->setCurrentItem(m_currentPage-1);
+        m_currentPage = m_doc->getNumPages()-1;
+        m_outputDev->setPage(m_currentPage+1);
+        pdfpartview->setCurrentItem(m_currentPage);
         updateActionPage();
     }
 }
@@ -287,19 +294,22 @@ void Part::slotGotoStart()
         m_currentPage = 0;
 
         m_outputDev->setPage(m_currentPage+1);
-        pdfpartview->pagesListBox->setCurrentItem(m_currentPage);
+        pdfpartview->setCurrentItem(m_currentPage);
         updateActionPage();
      }
 }
 
 bool Part::nextPage()
 {
-    m_currentPage = pdfpartview->pagesListBox->currentItem() + 1;
+    m_currentPage++;
     if ( m_doc && m_currentPage >= m_doc->getNumPages())
+    {
+        m_currentPage--;
         return false;
+    }
 
-    pdfpartview->pagesListBox->setCurrentItem(m_currentPage);
-    m_outputDev->nextPage();
+    m_outputDev->setPage(m_currentPage+1);
+    pdfpartview->setCurrentItem(m_currentPage);
     updateActionPage();
     return true;
 }
@@ -316,12 +326,15 @@ void Part::slotPreviousPage()
 
 bool Part::previousPage()
 {
-    m_currentPage = pdfpartview->pagesListBox->currentItem() - 1;
-    if ( m_currentPage  < 0)
+    m_currentPage--;
+    if (m_currentPage < 0)
+    {
+        m_currentPage++;
         return false;
+    }
 
-    pdfpartview->pagesListBox->setCurrentItem(m_currentPage );
-    m_outputDev->previousPage();
+    m_outputDev->setPage(m_currentPage+1);
+    pdfpartview->setCurrentItem(m_currentPage);
     updateActionPage();
     return true;
 }
@@ -341,6 +354,7 @@ Part::createAboutData()
   bool
 Part::closeURL()
 {
+  pdfpartview->stopThumbnailGeneration();
   delete m_doc;
   m_doc = 0;
 
@@ -355,7 +369,7 @@ Part::openFile()
   if (file.open(IO_ReadOnly) == false)
     return false;
 
-  GString* filename = new GString(m_file.ascii());
+  GString* filename = new GString( QFile::encodeName( m_file ) );
   m_doc = new PDFDoc(filename, 0, 0);
 
   if (!m_doc->isOk())
@@ -363,18 +377,17 @@ Part::openFile()
   // just for fun, set the status bar
   // emit setStatusBarText( QString::number( m_doc->getNumPages() ) );
 
-  // fill the listbox with entries for every page
-  pdfpartview->pagesListBox->setUpdatesEnabled(false);
-  pdfpartview->pagesListBox->clear();
-  for (int i = 1; i <= m_doc->getNumPages(); i++)
+  if (m_doc->getNumPages() > 0)
   {
-    pdfpartview->pagesListBox->insertItem(QString::number(i));
+    // TODO use a qvaluelist<int> to pass aspect ratio?
+    pdfpartview->setPages(m_doc->getNumPages(), m_doc->getPageHeight(1)/m_doc->getPageWidth(1));
+    pdfpartview->generateThumbnails(m_doc);
+
+    displayPage(1);
+    pdfpartview->setCurrentItem(0);
+    m_outputDev->setPDFDocument(m_doc);
   }
-  pdfpartview->pagesListBox->setUpdatesEnabled(true);
-  pdfpartview->pagesListBox->update();
 
-  displayPage(1);
-  m_outputDev->setPDFDocument(m_doc);
   return true;
 }
 
@@ -520,6 +533,7 @@ Part::print()
   printer.setPageSelection(KPrinter::ApplicationSide);
   printer.setMinMax(1, m_doc->getNumPages());
   printer.setCurrentPage(m_currentPage+1);
+  printer.setMargins(0, 0, 0, 0);
 
   if (printer.setup(widget()))
   {
@@ -637,12 +651,10 @@ Part::redrawPage()
 	displayPage(m_currentPage);
 }
 
-void Part::pageClicked ( QListBoxItem * qbi )
+void Part::pageClicked ( int i )
 {
-    if ( !qbi )
-        return;
-    m_currentPage = pdfpartview->pagesListBox->index(qbi)+1;
-    m_outputDev->setPage(m_currentPage);
+    m_currentPage = i;
+    m_outputDev->setPage(m_currentPage+1);
     updateActionPage();
 }
 
@@ -665,28 +677,51 @@ void Part::printPreview()
     return;
 
   KPrinter printer;
+  printer.setMargins(0, 0, 0, 0);
   printer.setPreviewOnly( true );
-  QPainter painter( &printer );
-  QOutputDevKPrinter printdev( painter, printer );
-  int max = m_doc->getNumPages();
-  for ( int i = 1; i <= max; ++i )
+  KTempFile tf( QString::null, ".ps" );
+  PSOutputDev *psOut = new PSOutputDev(tf.name().latin1(), m_doc->getXRef(), m_doc->getCatalog(), 1, m_doc->getNumPages(), psModePS);
+
+  if (psOut->isOk())
   {
-    m_doc->displayPage( &printdev, i, printer.resolution(), 0, true );
-    if ( i != max )
-      printer.newPage();
+    m_docMutex.lock();
+    m_doc->displayPages(psOut, 1, m_doc->getNumPages(), 72, 72, 0, globalParams->getPSCrop(), gFalse);
+    m_docMutex.unlock();
+
+    delete psOut;
+
+    printer.printFiles(tf.name(), true);
   }
+  else delete psOut;
 }
 
 void Part::doPrint( KPrinter& printer )
 {
-  QPainter painter( &printer );
-  QOutputDevKPrinter printdev( painter, printer );
-  QValueList<int> pages = printer.pageList();
-  for ( QValueList<int>::ConstIterator i = pages.begin(); i != pages.end();)
+  if (!m_doc->okToPrint())
   {
-    m_doc->displayPage( &printdev, *i, printer.resolution(), 0, true );
-    if ( ++i != pages.end() )
-      printer.newPage();
+    // No new messages allowed on branch
+    // KMessageBox::error(widget(), i18n("Printing this document is not allowed."));
+    return;
+  }
+  
+  KTempFile tf( QString::null, ".ps" );
+  PSOutputDev *psOut = new PSOutputDev(tf.name().latin1(), m_doc->getXRef(), m_doc->getCatalog(), 1, m_doc->getNumPages(), psModePS);
+  if (psOut->isOk())
+  {
+    std::list<int> pages;
+
+    QValueList<int> pageList = printer.pageList();
+    QValueList<int>::const_iterator it;
+
+    for(it = pageList.begin(); it != pageList.end(); ++it) pages.push_back(*it);
+    
+    m_docMutex.lock();
+    m_doc->displayPages(psOut, pages, 72, 72, 0, globalParams->getPSCrop(), gFalse);
+    m_docMutex.unlock();
+    
+    delete psOut;
+    
+    printer.printFiles(tf.name(), true);
   }
 }
 
